/* xCat.c -- Unpack file to stdout
 * Copyright (C) 1996-2000,2007 authors
 * This file is part of the xpk package.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 */

/*
 * Author: SDI (before 1.1 Urban Dominik Mller)
 * Written by Dirk Stcker <stoecker@amigaworld.com>
 * UNIX version by Vesa Halttunen <vesuri@jormas.com>
 * patched by Ralf Hoffmann <ralf@boomerangsworld.de>
 *   based on xType.c but doesn't add newlines at all
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <xpk/xpk.h>
#include <xpk/amigalibs.h>
#include <xpkmaster/xpkmaster.h>

#define XPKLEN_ONECHUNK 0x7fffffff

char errbuf[XPKERRMSGSIZE + 1], *outbuf;
int outbuflen;

void end( const char *text )
{
  if(text) fprintf(stderr, "%s\n", text);
  exit( text ? EXIT_FAILURE : EXIT_SUCCESS );
}

int main(int argc, char **argv)
{
  struct XpkBuffer *xfh;
  char *ptr;
  int i, len;

  if(argc < 2 || argv[1][0] == '?')
    end("Usage: xCat filename");

  for(i=1; i < argc; i++) {
    if(XpkOpen(&xfh, (struct TagItem[]){ { XPK_InName, { argv[i] } },
                                         { XPK_PassThru, { .ti_Data_u32 = 1 } },
                                         { TAG_DONE }  }))
      end(errbuf);
    if(!(outbuf=(char *) malloc(outbuflen=xfh->xb_Fib.xf_NLen + XPK_MARGIN)))
      end("Out of memory");
    while((len=XpkRead(xfh, outbuf, xfh->xb_Fib.xf_NLen)) > 0) {
      ptr=outbuf;
      fwrite(ptr, len, 1, stdout);
    }

    if(XpkClose(xfh) || len)
      end(errbuf);

    free(outbuf);
    outbuf=0;
  }

  end(0);
}
